"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MockStrategy = void 0;
exports.proxyLifetimes = proxyLifetimes;
exports.proxyLifetime = proxyLifetime;
exports.mockDependency = mockDependency;
var Errors_1 = require("../Errors");
var utils_1 = require("../utils");
var ServiceProvider_1 = require("../ServiceProvider");
var MockStrategy;
(function (MockStrategy) {
    /**
     * ONLY HERE FOR DOCUMENTATION PURPOSES, DO NOT USE IN CODE
     *
     * Default behavior for any dependency/dependency property given in the ProviderMock that isn't a MockStrategy
     * Getter usage will return the most recent dummy value (either the initial value, or a later set value)
     * Setter usage will set new dummy value
     */
    MockStrategy["dummyStub"] = "dummyStub";
    /**
     * This is the default behavior unless otherwise explicitly stated
     *
     * Getter usage will return null
     * Setter usage will do nothing
     */
    MockStrategy["nullStub"] = "nullStub";
    /**
     * Getter usage will throw NotMockedException(dependency, prop, usage-type)
     * Setter usage will throw NotMockedException(dependency, prop, usage-type)
     */
    MockStrategy["exceptionStub"] = "exceptionStub";
    /**
     * Returns the real value, anything with this strategy won't be mocked at all
     */
    MockStrategy["realValue"] = "realValue";
})(MockStrategy || (exports.MockStrategy = MockStrategy = {}));
function proxyLifetimes(services, providerMock, defaultMock) {
    if (defaultMock === void 0) { defaultMock = MockStrategy.nullStub; }
    if (typeof providerMock === 'string') {
        defaultMock = providerMock;
    }
    var dependencyMock = typeof providerMock === 'string' ? {} : providerMock;
    var provider = services.buildDebug();
    var lifetimes = Object.values(provider.lifetimes).map(function (lifetime) {
        var _a;
        var propertyMock = (_a = dependencyMock[lifetime.name]) !== null && _a !== void 0 ? _a : defaultMock;
        return proxyLifetime(lifetime, propertyMock, defaultMock);
    }).reduce(function (a, b) {
        a[b.name] = b;
        return a;
    }, {});
    return new ServiceProvider_1.DebugServiceProvider(lifetimes);
}
var _a = (0, utils_1.propertyOf)(), provide = _a.provide, validate = _a.validate;
function proxyLifetime(lifetime, dependencyMock, defaultMock) {
    var name = lifetime.name.toString();
    return new Proxy(lifetime, {
        get: function (target, prop) {
            if (prop !== provide && prop !== validate)
                return target[prop];
            return function (provider, context) {
                var shadow = target.provide(provider, context);
                var mock;
                switch (typeof dependencyMock) {
                    case 'string':
                        mock = mockDependency(name, shadow, {}, dependencyMock);
                        break;
                    case 'function':
                        mock = mockDependency(name, shadow, dependencyMock(provider.createProxy(context), undefined, provider, context), defaultMock);
                        break;
                    case 'object':
                        mock = mockDependency(name, shadow, dependencyMock, defaultMock);
                        break;
                    default:
                        mock = mockDependency(name, shadow, {}, defaultMock);
                        break;
                }
                if (name in provider.instances)
                    provider.instances[name] = mock;
                if (name in context.instances)
                    context.instances[name] = mock;
                return mock;
            };
        },
    });
}
function mockDependency(name, shadowInstance, propertyMock, defaultMock) {
    function isFunc(prop) {
        return typeof shadowInstance[prop] === 'function';
    }
    function mockValue(prop, mockType, type) {
        switch (mockType) {
            case MockStrategy.realValue:
                return shadowInstance[prop];
            case MockStrategy.exceptionStub:
                if (!isFunc(prop))
                    throw new Errors_1.ShouldBeMockedDependencyError(name, prop, type);
                return function () {
                    throw new Errors_1.ShouldBeMockedDependencyError(name, prop, 'function');
                };
            case MockStrategy.nullStub:
            default:
                return isFunc(prop) ? function () { return null; } : null;
        }
    }
    // noinspection JSUnusedGlobalSymbols
    var proxy = new Proxy(propertyMock, {
        get: function (target, prop) {
            var _a;
            var value = target[prop];
            var mockType = value;
            if (prop in target && mockType in MockStrategy) {
                return mockValue(prop, mockType, 'get');
            }
            else if (prop in target || ((_a = Object.getOwnPropertyDescriptor(target, prop)) === null || _a === void 0 ? void 0 : _a.get)) {
                return value;
            }
            else {
                return mockValue(prop, defaultMock, 'get');
            }
        },
        set: function (target, prop, newValue) {
            var _a;
            var value = target[prop];
            var mockType = value;
            if (prop in target && mockType in MockStrategy) {
                target[prop] = mockValue(prop, mockType, 'set');
            }
            else if (prop in target || ((_a = Object.getOwnPropertyDescriptor(target, prop)) === null || _a === void 0 ? void 0 : _a.set)) {
                target[prop] = newValue;
            }
            else {
                target[prop] = mockValue(prop, defaultMock, 'set');
            }
            return true;
        },
    });
    return proxy;
}
